-- $Rev: 28 $

local addonName, L = ...;
local BH_ProjectUrl = "http://www.curse.com/addons/wow/byehakmud";

--- Properties of Bye Hakmud

function BH_SetByeHakmundEnabled(enabled)
	local oldEnabled = BH_IsByeHakmudEnabled();
	BH_IsDismountHakmudEnabled = enabled;
	if (oldEnabled ~= enabled) then
		BH_SetListen4Vehicle(BH_IsByeHakmudEnabled());
		if (BH_IsByeHakmudEnabled()) then
			BH_PostMessage (L["ByeHakmud turned on: Vendor can be ejected when the mount is invoked"]);
		else
			BH_PostMessage (L["ByeHakmud turned off: Vendor stays on the mount"]);
		end
	end
end

function BH_IsHakmudKeptOnShift()
	local retVal = true;
	if (BH_HakmudKeptOnShift ~= nil) then
		retVal = BH_HakmudKeptOnShift;
	end
	return retVal;
end

function BH_SetHakmudKeptOnShift(keep)
	BH_HakmudKeptOnShift = keep;
end

function BH_IsOnlyEjectWhenInParty()
    local retVal = true;
    if (BH_OnlyEjectWhenInParty ~= nil) then
	retVal = BH_OnlyEjectWhenInParty;
    end
    return retVal;
end

function BH_SetOnlyEjectWhenInParty(EjectInParty)
    BH_OnlyEjectWhenInParty = EjectInParty
end

function BH_GetVersion()
	local version = "unversioned";
	local repoDate = "$Date: 2012-09-27 22:21:12 +0000 (Thu, 27 Sep 2012) $";
	local repoUrl = "$URL: file:///media/cf-repositories/svn/wow/byehakmud/mainline/tags/V%201.01.01/ByeHakmud.lua $";

	repoUrl = string.gsub(repoUrl, "%%20", " "); -- spaces are url encoded
	local tagPattern = "/tags/[^/]*";
	local tagStart, tagEnd = string.find(repoUrl, tagPattern);

	if (tagStart ~= nil and tagEnd ~= nil) then
		tagStart = tagStart + string.len("/tags/");
		version = string.sub(repoUrl, tagStart, tagEnd);
	else
		local datePattern = "$Date:[^$]*";
		local dateStart, dateEnd = string.find(repoDate, datePattern);
		if (dateStart ~= nil and dateEnd ~= nil) then
			dateStart = dateStart + string.len("$Date:");
			dateStr = string.sub(repoDate, dateStart, dateEnd);
			version = "unversioned "..tostring(dateStr);
		end
	end
	
	return version;
end 

 --- Config Panel View Accessors
 local BH_ConfigPanel;
 
 function BH_GetConfigPanel()
	return BH_ConfigPanel;
end
 
function BH_GetBHEnabledCB()
	local panel = BH_GetConfigPanel();
	return _G[panel:GetName().."_CB_ByeHakmudEnabled"];
end

function BH_GetBHOnlyEjectWhenInPartyCB()
	local panel = BH_GetConfigPanel();
	return _G[panel:GetName().."_CB_OnlyEjectWhenInGroup"];
end

--- Config Panel View Initialization

function BH_AddConfigPanelToAddonOptions()
	local panel = BH_GetConfigPanel();
	panel.name = "Bye Hakmud";
	panel.okay = function (self) BH_OnOk(); end;
	panel.refresh = function (self) BH_OnRefresh(); end;
	InterfaceOptions_AddCategory(panel);

        _G[panel:GetName().."_Description"]:SetText(TL(
            "ByeHakmud can automatically dismount a vendor when "..
            "a mount like the Traveler's Tundra Mammoth is invoked. "..
            "That can be useful if you want to keep a seat free for a "..
            "group member to join your mount and repair his armor while riding."
          ));
	  
	_G[panel:GetName().."_Version"]:SetText(BH_GetVersion() .."\n" .. BH_ProjectUrl);
end

function TL(s)
  return L[s];
end

--- Config Panel Controller

function BH_OnOk()
	local enabled = BH_GetBHEnabledCB():GetChecked() == 1;
	BH_SetByeHakmundEnabled(enabled);

	local onlyEjectWhenInParty = BH_GetBHOnlyEjectWhenInPartyCB():GetChecked() == 1;
	BH_SetOnlyEjectWhenInParty(onlyEjectWhenInParty);

	BH_SetHakmudKeptOnShift(BH_RB_Model_IsKeptOnShift());
end

function BH_OnRefresh()
	BH_GetBHEnabledCB():SetChecked(BH_IsByeHakmudEnabled());
	BH_GetBHOnlyEjectWhenInPartyCB():SetChecked(BH_IsOnlyEjectWhenInParty());
	BH_RB_Model_SetKeptOnShift(BH_IsHakmudKeptOnShift());
end

function BH_ConfigPanel_OnLoad(frame)
	BH_ConfigPanel = frame;
	BH_AddConfigPanelToAddonOptions();
	
	frame:RegisterEvent("ADDON_LOADED");
end

---  

function BH_IsByeHakmudEnabled()
	local retVal = true;
	if (BH_IsDismountHakmudEnabled ~= nil) then
		retVal = BH_IsDismountHakmudEnabled;
	end
	return retVal;
end

--- Controller of Radio Button 'Hakmud kept on <shift>'
local BH_RB_Model_KeptOnShift;

function BH_RB_Model_IsKeptOnShift()
	return BH_RB_Model_KeptOnShift;
end

function BH_RB_Model_SetKeptOnShift(keep)
	BH_RB_Model_KeptOnShift = keep;
	BH_UpdateHakmudKeptOnShiftButtons();
end

function BH_UpdateHakmudKeptOnShiftButtons()
	local keptOnShift = BH_RB_Model_IsKeptOnShift();
	_G["ByeHakmudConfigFrame_RB_SHIFT_KEEPS"]:SetChecked(keptOnShift);
	_G["ByeHakmudConfigFrame_RB_SHIFT_KICKS"]:SetChecked(not keptOnShift);
end

--- Bye Hakmud Engine

function BH_SetListen4Event(enabled, eventName)
	if (enabled) then
		if (not BH_GetConfigPanel():IsEventRegistered(eventName)) then
			BH_GetConfigPanel():RegisterEvent(eventName); 
		end
	else 
		if (BH_GetConfigPanel():IsEventRegistered(eventName)) then
			BH_GetConfigPanel():UnregisterEvent(eventName); 
		end
	end
end

function BH_SetListen4Vehicle(enabled)
	BH_SetListen4Event(enabled, "PLAYER_GAINS_VEHICLE_DATA");
end

function BH_SetListen4Passenger(enabled)
	BH_SetListen4Event(enabled, "VEHICLE_PASSENGERS_CHANGED");
end

function BH_IsPartyAllowsEject()
	local retVal = true;
	if BH_IsOnlyEjectWhenInParty() then
	    retVal = UnitInParty("player")
	end
	return retVal;
end

function BH_EjectHakmud()
	function BH_GetSeatsToEject()
		local retVal = nil;
		local companionCount = GetNumCompanions("MOUNT");
		for i = 1, companionCount do
			local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
			if issummoned then
			    retVal = BH_MammothIDList[creatureID];
			    break;
			end
		end
		return retVal;
	end

	local keepHakmud = IsShiftKeyDown();
	if (not BH_IsHakmudKeptOnShift()) then
		keepHakmud = not keepHakmud;
	end
	
	if ((not keepHakmud)
	    and BH_IsPartyAllowsEject())
	then 
	    BH_SetListen4Passenger(false);
	    local seatsToEject = BH_GetSeatsToEject();
	    if seatsToEject then
		local ejected = false
		for i, seat in ipairs(seatsToEject) do
		    if CanEjectPassengerFromSeat(seat) then
			EjectPassengerFromSeat(seat)
			ejected = true
		    end
		end
		
		if (ejected) then
		    BH_PostMessage(L["ByeHakmud ejected vendor from the mount"])
		end
	    end
	end
end

--- Bye Hakmud Engines Event Handlers

function BH_OnAddonLoaded(addonName)
	if (addonName == "ByeHakmud") then
		BH_SetListen4Vehicle(BH_IsByeHakmudEnabled());
		BH_GetConfigPanel():UnregisterEvent("ADDON_LOADED");
	end
end

function BH_OnVehiclePassengersChanged()
	BH_EjectHakmud() ;
end

function BH_OnPlayerGainsVehicleData()
	BH_SetListen4Passenger(true);
end

function BH_OnEvent(self, event, arg1, ...)
  if      (event == "ADDON_LOADED") then BH_OnAddonLoaded(arg1);
  elseif (event == "VEHICLE_PASSENGERS_CHANGED") then BH_OnVehiclePassengersChanged();
  elseif (event == "PLAYER_GAINS_VEHICLE_DATA") then BH_OnPlayerGainsVehicleData();
  end
end

--- Command line interface

SLASH_BYEHAKMUD1 = '/ByeHakmud';
function SlashCmdList.BYEHAKMUD(msg, editbox)
	if msg == 'on' then
		BH_SetByeHakmundEnabled(true);
	elseif msg == 'off' then
		BH_SetByeHakmundEnabled(false);
	elseif msg == 'toggle' then
		BH_SetByeHakmundEnabled(not BH_IsByeHakmudEnabled());
	elseif msg == 'keep' then
		BH_SetHakmudKeptOnShift(true);
	elseif msg == 'kick' then
		BH_SetHakmudKeptOnShift(false);
	elseif msg == 'gui' then
		InterfaceOptionsFrame_OpenToCategory(ByeHakmudConfigFrame);
	elseif msg == 'eject' then
		BH_EjectHakmud();
	else
		BH_PostMessage(
			"ByeHakmud Version: " .. BH_GetVersion() .. " " .. BH_ProjectUrl .."\n" ..
			L["/ByeHakmud on -- automatic ejection enabled\n" ..
			  "/ByeHakmud off -- automatic ejection turned off\n"..
			  "/ByeHakmud toggle -- switch automatich ejection on / off\n"..
			  "/ByeHakmud keep -- holding <shift> keeps Hakmud on the mount\n"..
			  "/ByeHakmud kick -- holding <shift> ejects Hakmud from the mount\n"..
			  "/ByeHakmud gui -- show configuration panel"
			]
		);
	end
end

function BH_PostMessage(msg)
	print(msg);
end